clc
close all
clear all
addpath src
addpath src/iso2mesh

% code for mapping the ground truth meshes from the native space
% (Cartesian) to polar. Most useful for RF based tracking. 

%%%% Parameters to set
seq_id = 'normal';
db_range_visu = 65; % desired image contrast in dB
%%%% END parameters to set

% -- probe setings
switch seq_id
    case {'ladprox','laddist','normal','lcx','rca'}
        load 'probe3D_3.3MHz_Journal.mat'
    case {'lbbb','lbbbsmall','sync'}
        load 'probe3D_3.3MHz_Journal_LBBB'
end
sys.dist_to_probe = 0.02;

% -- load rf
rf_fname = ['data\rf\' seq_id '\rf0.mat'];
load(rf_fname)

% -- compute envelope
n_sub = floor(size(rf,2) / (sys.res_axial + 1));
sys.nr_lines_elev = size(rf,3);
env_sim = HlibSubsampl(rf,n_sub,sys);

% -- envelope in polar
rho = linspace(sys.start_depth,sys.end_depth,size(env_sim,2));
theta = linspace(-sys.width/2,sys.width/2,size(env_sim,1));
phi = linspace(-sys.width_elev/2,sys.width_elev/2,size(env_sim,3)); 

logenv_sim = env_sim / max(env_sim(:));
logenv_sim = 20*log10(logenv_sim) + db_range_visu;
logenv_sim(logenv_sim<0) = 0;
logenv_sim = logenv_sim * 236 / db_range_visu;

% -- load ground truth in cartesian
fname = ['data\ground_truth\' seq_id '\simu___0.vtk']; 
[mesh] = read_vtkMesh_martino(fname);

% -- load bmode
fname = ['data\bmode\' seq_id '\frame0.mhd']; 
[img,info] = read_mhd(fname);

% -- cartesian domain
elev = info.Dimensions(1)*info.PixelDimensions(1)/1000;
width = info.Dimensions(2)*info.PixelDimensions(2)/1000;
radius = info.Dimensions(3)*info.PixelDimensions(3)/1000;
fov = [0 radius -width/2 width/2 -elev/2 elev/2];

% -- CONVERT GROUND TRUTH TO POLAR
z_point = mesh.points(:,3);
x_point = mesh.points(:,1) - 1000*elev/2;
y_point = mesh.points(:,2) - 1000*width/2;
[th_point,phi_point,r_point] = cart2sph(z_point,y_point,x_point);
r_point = r_point/1000 + sys.dist_to_probe;

figure(1)
slice(rho,theta,phi,double(logenv_sim),mean(rho),mean(theta),mean(phi)), shading flat,
colormap gray, 
axis tight
xlabel('rho')
ylabel('theta')
zlabel('phi')
hold on,
plot3(r_point,th_point,phi_point,'.r');
hold off

% -- scan convert image
x = 1000*linspace(0,elev,info.Dimensions(1));
y = 1000*linspace(0,width,info.Dimensions(2));
z = 1000*linspace(0,radius,info.Dimensions(3));

% -- desired image size
nfinal = info.Dimensions(2); % desired final size after scan conversion 
mfinal = info.Dimensions(1);
pfinal = info.Dimensions(3);
 
% -- zoom polar space to tissue region
x0 = linspace(sys.start_depth,sys.end_depth,size(env_sim,2));
xi = linspace(sys.start_depth+sys.dist_to_probe,sys.end_depth+sys.dist_to_probe,size(env_sim,2));

tmp = permute(logenv_sim,[2 1 3]);
logenv_simi = interp1(x0,tmp,xi,'linear',0); 
logenv_simi = ipermute(logenv_simi,[2 1 3]);

% -- scan conversion (takes some time)
sys.nr_lines = size(logenv_simi,1);
disp('Scan conversion ...')
sec_sim = ScanConvert(logenv_sim,sys,fov);
sec_sim(isnan(sec_sim)) = 0;
sec_sim = permute(sec_sim,[1 3 2]);

% -- resample to final size
[nsim,msim,psim] = size(sec_sim);
zsim = linspace(0,radius,psim);
ysim = linspace(-width/2,width/2,nsim);
xsim = linspace(-elev/2,elev/2,msim);

[xi,yi,zi] = meshgrid(1:mfinal,1:nfinal,1:pfinal);
xi = (xi - 1)/(mfinal - 1) * (msim - 1) + 1;
yi = (yi - 1)/(nfinal - 1) * (nsim - 1) + 1;
zi = (zi - 1)/(pfinal - 1) * (psim - 1) + 1;
sec_sim = interp3(double(sec_sim),xi,yi,zi,'linear',0);

% -- CONVERT GROUND TRUTH FROM POLAR TO CARTESIAN.  
[z_point1,y_point1,x_point1] = sph2cart(th_point,phi_point,(r_point-sys.dist_to_probe)*1000);
x_point1 = x_point1 + 1000*elev/2;
y_point1 = y_point1 + 1000*width/2;

% error introduced by the scan conversion
err = sum(([x_point1,y_point1,z_point1] - mesh.points).^2,2);

[n,m,p] = size(sec_sim);
slice(x,y,z,double(sec_sim),mean(x),mean(y),mean(z)), shading flat, axis image
colormap gray, hold on,
plot3(mesh.points(:,1),mesh.points(:,2),mesh.points(:,3),'.r')
plot3(x_point1,y_point1,z_point1,'og')
xlabel('x [mm]')
ylabel('y [mm]')
zlabel('z [mm]')

% -- 
figure
scatter3(x_point1,y_point1,z_point1,5,err,'filled'), axis image, colorbar
